function sfun_ram(block)
%
% S-Funktion fuer einen vorsetzbaren RAM-Speicher-Baustein 1k x 16
%
% Eingaenge:
%    Dateneingang (16 bit)
%    Adresseingang (10 bit)
%    read/write flag
%
% Ausgang:
%    Datenausgang (16 bit)
%
% Interne Zustaende
%    x    eigentlicher Speicher 
%
% externe Parameter:
%    useFiles   1 -> Speicher wird am Anfangs aus ram.in gelesen,
%               am Ende nach ram.out geschrieben (als Liste von Dezimalzahlen)
%
% Funktion:
%   r/w = 0 -> Ausgang = unter Adresse gespeicherter Wert
%            Dateneingang wird ignoriert
%   r/w = 1 -> Dateneingang wird an Adresse gespeichert
%            Ausgang = 0
  setup(block); 
end

%% -------------------------------------------------------------------
function setup(block)
  block.NumInputPorts  = 3;
  block.NumOutputPorts = 1;
  block.NumDialogPrms  = 1;

  block.SetPreCompInpPortInfoToDynamic;  % properties are inherited
  block.SampleTimes = [-1 0];   % inherited
  
  block.RegBlockMethod('PostPropagationSetup', @DoPostPropSetup);
  block.RegBlockMethod('InitializeConditions', @InitializeConditions);
  block.RegBlockMethod('Update', @Update);
  block.RegBlockMethod('Outputs', @Outputs);
  block.RegBlockMethod('Terminate', @Terminate);
end

%% -------------------------------------------------------------------
function DoPostPropSetup(block)
  block.NumDworks = 1;
  block.Dwork(1).Name = 'x0'; 
  block.Dwork(1).Dimensions      = 1024;
  block.Dwork(1).DatatypeID      = 0;
  block.Dwork(1).Complexity      = 'Real';
  block.Dwork(1).UsedAsDiscState = true;
end
    
%% -------------------------------------------------------------------
function InitializeConditions(block)
  useFiles = block.DialogPrm(1).Data;
  
  ram  = zeros(1024, 1);
  if useFiles == 1 && exist('ram.in', 'file') == 2
    ram = load('ram.in');
  end
  block.Dwork(1).Data = ram;
end

%% -------------------------------------------------------------------
function Update(block)
  data_in = block.InputPort(1).Data;
  address = block.InputPort(2).Data + 1;
  r_w     = block.InputPort(3).Data;

  memsize = block.Dwork(1).Dimensions;
  if ((address > memsize) || (address <= 0))
     error('memory index out of bounds'); 
  end

  if r_w == 1                   % write
    block.Dwork(1).Data(address) = data_in;
  end
end

%% -------------------------------------------------------------------
function Outputs(block)
  address = block.InputPort(2).Data + 1;
  r_w     = block.InputPort(3).Data;

  memsize = block.Dwork(1).Dimensions;
  if ((address > memsize) || (address <= 0))
     error('memory index out of bounds'); 
  end

  if r_w == 0  % read
     block.OutputPort(1).Data = block.Dwork(1).Data(address);
  else
     block.OutputPort(1).Data = 0.0;      
  end
end

%% -------------------------------------------------------------------
function Terminate(block)
  useFiles = block.DialogPrm(1).Data;
  ram = block.Dwork(1).Data;
  
  if useFiles == 1
    fid = fopen('ram.out', 'w');
    fprintf(fid, '%5d\n', ram);
    fclose(fid);
  end
end